`timescale 100 ns / 1 ns
// All cells assume preset and clear are active low

// This dff cell has a reset priority over preset


primitive udp_dff_p0(q, d, clk,clear, preset);

	output q;
	input clk, clear, d, preset;
	reg q; /* declaring output as reg*/
	

	table
	// d 	clk	clear	preset	:	state	: q
	   ?	?	0  	?	:	?    	: 0;// if clear, set out to 0              
	   ?	?	1  	0	:	?    	: 1;// if not clear, preset to 1           
	   0    r	1  	1	:	?    	: 0;// on rising edge transmit data         
	   1	r	1  	1	: 	?    	: 1;// "                                    
	   ?	n	1  	1	:	?    	: -;// on negedge, do nothing               
	   *	?	1  	1	:	?    	: -;// on steady clock, ignore data trans.  
	   ?	?	p  	1	:	?    	: -;// ignore positive edge of clear        
	   ?	?	1  	p	:	?    	: -;// ignore positive edge of preset
	endtable
endprimitive


module dff_p0(q, d, clk,clear, preset);

	output q;
	input clk, clear, d, preset;
  udp_dff_p0 D1(q, d, clk,clear, preset);

endmodule // dff_p0

// This cell has a preset priority over reset

primitive udp_dff_p1(q, d, clk,clear, preset);

	output q;
	input clk, clear, d, preset;
	reg q; /* declaring output as reg*/
	

	table
	// d 	clk	clear	preset	:	state	: q
	   ?	?	?  	0	:	?    	: 1;// if preset, set out to 1
	   ?	?	0  	1	:	?    	: 0;// if not preset, clear to 0
	   0    r	1  	1	:	?    	: 0;// on rising edge transmit data
	   1	r	1  	1	: 	?    	: 1;// "
	   ?	n	1  	1	:	?    	: -;// on negedge, do nothing
	   *	?	1  	1	:	?    	: -;// on steady clock, ignore data trans.
	   ?	?	p  	1	:	?    	: -;// ignore positive edge of clear        
	   ?	?	1  	p	:	?    	: -;// ignore positive edge of preset
	endtable
endprimitive


module dff_p1(q, d, clk,clear, preset);

	output q;
	input clk, clear, d, preset;
  udp_dff_p1 D1(q, d, clk,clear, preset);

endmodule // dff_p1

primitive udp_dff_px(q, d, clk,clear, preset);

	output q;
	input clk, clear, d, preset;
	reg q; /* declaring output as reg*/
	

	table
	// d 	clk	clear	preset	:	state	: q
	   ?	?	0  	0	:	?    	: x;// if clear, set out to 0              
	   ?	?	0  	1	:	?    	: 0;// if clear, set out to 0              
	   ?	?	1  	0	:	?    	: 1;// if not clear, preset to 1           
	   0    r	1  	1	:	?    	: 0;// on rising edge transmit data         
	   1	r	1  	1	: 	?    	: 1;// "                                    
	   ?	n	1  	1	:	?    	: -;// on negedge, do nothing               
	   *	?	1  	1	:	?    	: -;// on steady clock, ignore data trans.  
	   ?	?	p  	1	:	?    	: -;// ignore positive edge of clear        
	   ?	?	1  	p	:	?    	: -;// ignore positive edge of preset
	endtable
endprimitive


module dff_px(q, d, clk,clear, preset);

	output q;
	input clk, clear, d, preset;
  udp_dff_px D1(q, d, clk,clear, preset);

endmodule // dff_px

primitive udp_ldlatch_p1(q, d, en, clear, preset);

	output q;
	input clear, preset, d, en;
	reg q;

	table
	// d	en 	clear	preset	:  state	: q
	   ?	?	?	0	:    ?		: 1;// preset to 1	
	   ?	?	0	1	:    ?		: 0;// clear to 0
	   0	1	1	1	:    ?		: 0;// on enable, transmit d
	   1	1	1	1	:    ?		: 1;// on enable, transmit d
	   ?    0	1	1	:    ?		: -;// not enable, no change
//	   *    0	1	1	:    ?		: -;// not enable, ignore d edges
	   ?	?	p  	1	:	?    	: -;// ignore positive edge of clear        
	   ?	?	1  	p	:	?    	: -;// ignore positive edge of preset   
	endtable
endprimitive


module ldlatch_p1(q, d, en, clear, preset);

	output q;
	input clear, preset, d, en;

   udp_ldlatch_p1 P1 (q, d, en, clear, preset);

endmodule // ldlatch_p1

primitive udp_ldlatch_px(q, d, en, clear, preset);

	output q;
	input clear, preset, d, en;
	reg q;

	table
	// d	en 	clear	preset	:  state	: q
	   ?	?	0	0	:    ?		: x;// indeterminate	
	   ?	?	1	0	:    ?		: 1;// preset to 1	
	   ?	?	0	1	:    ?		: 0;// clear to 0
	   0	1	1	1	:    ?		: 0;// on enable, transmit d
	   1	1	1	1	:    ?		: 1;// on enable, transmit d
	   ?    0	1	1	:    ?		: -;// not enable, no change
//	   *    0	1	1	:    ?		: -;// not enable, ignore d edges
	   ?	?	p  	1	:	?    	: -;// ignore positive edge of clear        
	   ?	?	1  	p	:	?    	: -;// ignore positive edge of preset   
	endtable
endprimitive


module ldlatch_px(q, d, en, clear, preset);

	output q;
	input clear, preset, d, en;

   udp_ldlatch_px P1 (q, d, en, clear, preset);

endmodule // ldlatch_px

primitive udp_ldlatch_p0(q, d, en, clear, preset);

	output q;
	input d, en, clear, preset;
	reg q;

	table
	// d	en 	clear	preset	:  state	: q
	   ?	?	0	?	:    ?		: 0;// clear to 0	
	   ?	?	1	0	:    ?		: 1;// preset to 1
	   0	1	1	1	:    ?		: 0;// on enable, transmit d
	   1	1	1	1	:    ?		: 1;// on enable, transmit d
	   ?    0	1	1	:    ?		: -;// not enable, no change
	   ?	?	p  	1	:	?    	: -;// ignore positive edge of clear        
	   ?	?	1  	p	:	?    	: -;// ignore positive edge of preset   
	endtable
endprimitive

module ldlatch_p0(q, d, en, clear, preset);

	output q;
	input clear, preset, d, en;

   udp_ldlatch_p0 P1 (q, d, en, clear, preset);

endmodule // ldlatch_p0


primitive udp_jk_ff_sr1 (q, ck, j, k, preset,clear);
	output q;
	input ck, j,k,preset,clear;
	reg q;

	table
     // ck	j	k	p	c 	: state	: q
//preset logic
	?	?	?	0	?	:  ?	: 1;
	?	?	?	*	1	:  1	: 1;
// clear logic
	?	?	?	1	0	:  ?	: 0;
	?	?	?	1	*	:  0	: 0;
// normal clocking cases
//	r	0	0	0	0	:  0	: 1;// check this up
	r	0	0	1	1 	:  ?	: -;
	r	0	1	1	1	:  ?	: 0;
	r	1 	0	1	1	:  ?	: 1;
	r	1	1	1	1	:  0	: 1;
	r 	1	1	1	1	:  1	: 0;
	f	?	?	?	?	:  ?	: -; //ignore falling edges

// j and k transitions
	b	*	?	?	?	:  ?	: -;
	b	?	*	?	?	:  ?	: -;

	p	0	0	1	1	:  ?	: -;
	p	0	?	1	?	:  0	: -;
	p	?	0	?	1	:  1	: -;
	(?0)	?	?	?	?	:  ?	: -;
	(1x)	0	0	1	1	:  ?	: -;
	(1x)	0	?	1	?	:  0	: -;
	(1x)	?	0	?	1	:  1	: -;
	x	*	0	?	1	:  1	: -;
	x	?	*	1	?	:  0	: -;
	endtable
endprimitive



module jk_ff_sr1 (q, ck, j, k, preset,clear);
   output q;			// 
   input       ck, j,k,preset,clear;//
   
   udp_jk_ff_sr1 I1 (q, ck, j, k, preset,clear);

endmodule // jk_ff_sr1



primitive udp_jk_ff_sr0 (q, ck, j, k, preset,clear);
	output q;
	input ck, j,k,preset,clear;
	reg q;

	table
     // ck	j	k	p	c 	: state	: q
// clear logic
	?	?	?	?	0	:  ?	: 0;
	?	?	?	1	*	:  0	: 0;
//preset logic
	?	?	?	0	1	:  ?	: 1;
	?	?	?	*	1	:  1	: 1;

// normal clocking cases
//	r	0	0	0	0	:  0	: 1;// check this up
	r	0	0	1	1 	:  ?	: -;
	r	0	1	1	1	:  ?	: 0;
	r	1 	0	1	1	:  ?	: 1;
	r	1	1	1	1	:  0	: 1;
	r 	1	1	1	1	:  1	: 0;
	f	?	?	?	?	:  ?	: -; //ignore falling edges

// j and k transitions
	b	*	?	?	?	:  ?	: -;
	b	?	*	?	?	:  ?	: -;

	p	0	0	1	1	:  ?	: -;
	p	0	?	1	?	:  0	: -;
	p	?	0	?	1	:  1	: -;
	(?0)	?	?	?	?	:  ?	: -;
	(1x)	0	0	1	1	:  ?	: -;
	(1x)	0	?	1	?	:  0	: -;
	(1x)	?	0	?	1	:  1	: -;
	x	*	0	?	1	:  1	: -;
	x	?	*	1	?	:  0	: -;
	endtable
endprimitive


module jk_ff_sr0 (q, ck, j, k, preset,clear);
   output q;			// 
   input       ck, j,k,preset,clear;//
   
   udp_jk_ff_sr0 I1 (q, ck, j, k, preset,clear);

endmodule // jk_ff_sr0


primitive udp_inv_clr0 (qn, clr, pre, inp);
	output qn;
	input clr, pre, inp;

	table
	//  	clr 	pre 	inp	: qn
		0	0	?	: 0;
		1	?	0	: 1;
		1	?	1	: 0;
		?	1	0	: 1;
		?	1	1	: 0;
		x	x	1	: 0;
		x	x	0	: 1;
	endtable
endprimitive


module inv_clr0 (qn, clr, pre, inp);
   output qn;			// 
   input  clr, pre, inp;	// 

   udp_inv_clr0 (qn, clr, pre, inp);

endmodule // inv_clr0

primitive udp_inv_clr1 (qn, clr, pre, inp);
	output qn;
	input clr, pre, inp;

	table
	//  	clr 	pre 	inp	: qn
		0	0	?	: 1;
		1	?	0	: 1;
		1	?	1	: 0;
		?	1	0	: 1;
		?	1	1	: 0;
		x	x	1	: 0;
		x	x	0	: 1;
	endtable
endprimitive
		
module inv_clr1 (qn, clr, pre, inp);
   output qn;			// 
   input  clr, pre, inp;	// 
   udp_inv_clr1 m1(qn, clr, pre, inp);

endmodule // inv_clr1
